/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapDispatch = wrapDispatch;

var _uiMetrics = require("@servicenow/ui-metrics");

var _constants = require("../../constants");

var _symbols = require("../../symbols");

var _utils = require("../../utils");

// this is done so that we don't break anyone dispatching from a disconnect
// function or action handler; people are relying on this to unsubscribe from
// AMB and other types of clean up. the AMB effect should be changed into a
// behavior that has a COMPONENT_DISCONNECTED handler, so that people don't
// have to manually clean up or rely on this broken functionality.
function wrapDispatch(host, action) {
  var hostVariables = (0, _utils.getHostVariables)(host);
  var interactionId = (0, _uiMetrics.getInteractionId)(action.meta);
  var helpers = action.meta[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] ? hostVariables[_symbols.HELPERS_IMMEDIATE] : hostVariables[_symbols.HELPERS_SCHEDULED];
  var originalUpdateState = helpers.updateState,
      originalUpdateProperties = helpers.updateProperties,
      originalUpdateContext = helpers.updateContext,
      originalDispatch = helpers.dispatch;

  var updateState = function updateState() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return originalUpdateState.apply(void 0, args.concat([interactionId]));
  };

  var updateProperties = function updateProperties() {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return originalUpdateProperties.apply(void 0, args.concat([interactionId]));
  };

  var updateContext = function updateContext() {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return originalUpdateContext.apply(void 0, args.concat([interactionId]));
  };

  var dispatch;

  if (action.type !== _constants.COMPONENT_DISCONNECTED && !action.meta[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED]) {
    dispatch = function dispatch(type, payload) {
      var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var error = arguments.length > 3 ? arguments[3] : undefined;
      // eslint-disable-next-line no-param-reassign
      meta = (0, _utils.normalizeActionMeta)(meta, {
        host: host
      });
      (0, _uiMetrics.setInteractionId)(meta, interactionId);
      return originalDispatch(type, payload, meta, error);
    };
  } else {
    dispatch = function dispatch(type, payload) {
      var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var error = arguments.length > 3 ? arguments[3] : undefined;
      // eslint-disable-next-line no-param-reassign
      meta = (0, _utils.normalizeActionMeta)(meta, {
        host: host
      });
      (0, _uiMetrics.setInteractionId)(meta, interactionId);
      meta[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] = true; // eslint-disable-line no-param-reassign

      return originalDispatch(type, payload, meta, error);
    };
  }

  dispatch.updateState = updateState;
  dispatch.updateProperties = updateProperties;
  dispatch.updateContext = updateContext;
  return {
    updateState: updateState,
    updateProperties: updateProperties,
    updateContext: updateContext,
    dispatch: dispatch
  };
}
//# sourceMappingURL=wrapDispatch.js.map