/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = debounce;

var _uiInternal = require("@servicenow/ui-internal");

var _hostMap = _interopRequireDefault(require("./hostMap"));

var _log = _interopRequireDefault(require("../../log"));

var _uiMetrics = require("@servicenow/ui-metrics");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function createDebounce(fn, _ref, host, ctx, runEffectFn) {
  var _ref$delay = _ref.delay,
      delay = _ref$delay === void 0 ? 0 : _ref$delay,
      maxWait = _ref.maxWait,
      _ref$collate = _ref.collate,
      collate = _ref$collate === void 0 ? false : _ref$collate;
  var debounceVariables = getDebounceVariables();
  var validMaxWait = Number.isInteger(maxWait);
  var shouldCollate = collate === true;
  var tDelay = !Number.isInteger(delay) ? 0 : delay;
  var tMaxWait = validMaxWait ? maxWait : 0;
  var componentName = host === null || host === void 0 ? void 0 : host.tagName;
  (0, _uiMetrics.logTimingMetrics)(_uiMetrics.timingMetricTypes.DEBOUNCE, tDelay, componentName);

  if (validMaxWait && tMaxWait < tDelay) {
    (0, _log["default"])('For the debounce modifier configurations, the property maxWait should be greater than the property delay.', {
      level: _uiInternal.allLogTypes.WARN,
      host: host,
      origin: 'debounce'
    });
  }

  var finishDebounce = function finishDebounce(fn) {
    invokeFn(shouldCollate, debounceVariables, runEffectFn, fn, ctx);
    debounceVariables = getDebounceVariables();
  };

  var collateData = function collateData(coeffects) {
    var _coeffects$action$pay = coeffects.action.payload,
        payload = _coeffects$action$pay === void 0 ? {} : _coeffects$action$pay;
    debounceVariables.collated.push({
      action: {
        payload: payload
      }
    });
  };

  return function debouncedEffect() {
    clearTimeout(debounceVariables.delayId);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    debounceVariables.lastArgs = args;
    debounceVariables.saveEventProps();
    if (shouldCollate) collateData(args[args.length - 1]);
    debounceVariables.delayId = (0, _uiInternal.defer)(function () {
      return finishDebounce(fn);
    }, tDelay);
    if (validMaxWait && tMaxWait && !debounceVariables.maxWaitId) debounceVariables.maxWaitId = (0, _uiInternal.defer)(function () {
      return finishDebounce(fn);
    }, tMaxWait);
  };
}

var invokeFn = function invokeFn(accumulate, debounceVariables, runEffectFn, fn, ctx) {
  debounceVariables.retrieveEventProps();
  var collated = debounceVariables.collated,
      lastArgs = debounceVariables.lastArgs;
  clearTimers(debounceVariables);

  if (accumulate) {
    var coeffects = lastArgs[lastArgs.length - 1];
    coeffects.collated = collated;
  }

  runEffectFn(fn, lastArgs, ctx);
};

var clearTimers = function clearTimers(_ref2) {
  var delayId = _ref2.delayId,
      maxWaitId = _ref2.maxWaitId;
  clearTimeout(delayId);
  clearTimeout(maxWaitId);
};

var getDebounceVariables = function getDebounceVariables() {
  return {
    delayId: null,
    maxWaitId: null,
    lastArgs: null,
    collated: [],
    event: {
      eventPath: [],
      currentTarget: null
    },
    saveEventProps: function saveEventProps() {
      var _coeffects$action;

      var coeffects = this.lastArgs[this.lastArgs.length - 1];
      var event = (_coeffects$action = coeffects.action) === null || _coeffects$action === void 0 ? void 0 : _coeffects$action.payload.event;
      this.event.eventPath = event === null || event === void 0 ? void 0 : event.composedPath();
      this.event.currentTarget = event === null || event === void 0 ? void 0 : event.currentTarget;
    },
    retrieveEventProps: function retrieveEventProps() {
      var _coeffects$action2,
          _this = this;

      var coeffects = this.lastArgs[this.lastArgs.length - 1];
      var event = (_coeffects$action2 = coeffects.action) === null || _coeffects$action2 === void 0 ? void 0 : _coeffects$action2.payload.event;
      if (typeof event === 'undefined') return;

      event.composedPath = function () {
        return _this.event.eventPath;
      };

      Object.defineProperty(event, 'currentTarget', {
        value: this.event.currentTarget,
        writable: false
      });
    }
  };
};
/**
 * Debounce modifier for effect functions
 *
 * @param host
 * @param actionType
 * @param effectFn
 * @param modifier
 */


function debounce(host, actionType, effectFn, modifier, ctx, runEffectFn) {
  if (!_hostMap["default"].has(host)) _hostMap["default"].set(host, new WeakMap());

  var effectMap = _hostMap["default"].get(host);

  if (!effectMap.has(effectFn)) effectMap.set(effectFn, {});
  var actionMappings = effectMap.get(effectFn);
  if (!actionMappings[actionType]) actionMappings[actionType] = createDebounce(effectFn, modifier, host, ctx, runEffectFn);
  return actionMappings[actionType];
}
//# sourceMappingURL=debounce.js.map