/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lifeCycleFnsToHandlers = lifeCycleFnsToHandlers;

var _normalizeActionHandler = require("./normalizeActionHandler");

var _uiInternal = require("@servicenow/ui-internal");

var _log = _interopRequireDefault(require("../../log"));

var _constants = require("../../constants");

var _getHostVariables = _interopRequireDefault(require("../getHostVariables"));

var _symbols = require("../../symbols");

var _utils = require("../../actions/utils");

var _lifeCycleFnsToHandle;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var LIFECYCLE = _uiInternal.locations.LIFECYCLE;
var lifeCycleFnsToHandlerMap = (_lifeCycleFnsToHandle = {}, _defineProperty(_lifeCycleFnsToHandle, _constants.COMPONENT_CONNECTED, {
  lifecycle: _constants.ON_CONNECT_LIFECYCLE,
  display: 'COMPONENT_CONNECTED'
}), _defineProperty(_lifeCycleFnsToHandle, _constants.COMPONENT_DISCONNECTED, {
  lifecycle: _constants.ON_DISCONNECT_LIFECYCLE,
  display: 'COMPONENT_DISCONNECTED'
}), _defineProperty(_lifeCycleFnsToHandle, _constants.COMPONENT_BOOTSTRAPPED, {
  lifecycle: _constants.ON_BOOTSTRAP_LIFECYCLE,
  display: 'COMPONENT_BOOTSTRAPPED'
}), _defineProperty(_lifeCycleFnsToHandle, _constants.COMPONENT_PROPERTY_CHANGED, {
  lifecycle: _constants.PROPERTIES_SET_LIFECYCLE,
  display: 'COMPONENT_PROPERTY_CHANGED'
}), _lifeCycleFnsToHandle);

function createOnPropertyChangeHandler(config, def) {
  var properties = config.properties;
  var onPropertiesSet = config[def.lifecycle];
  var fns = {};

  for (var k in properties) {
    var onChange = properties[k].onChange;
    if (typeof onChange === 'function') fns[k] = onChange;
  }

  if (!Object.keys(fns).length && !onPropertiesSet) return;
  return function onPropertyChange(_ref) {
    var dispatch = _ref.dispatch,
        _ref$action$payload = _ref.action.payload,
        host = _ref$action$payload.host,
        name = _ref$action$payload.name,
        previousValue = _ref$action$payload.previousValue,
        value = _ref$action$payload.value;
    var hostVariables = (0, _getHostVariables["default"])(host);

    if (onPropertiesSet) {
      (0, _uiInternal.sandbox)(onPropertiesSet, {
        args: [host, hostVariables[_symbols.HELPERS_SCHEDULED]],
        dispatch: dispatch,
        host: host,
        location: LIFECYCLE,
        details: {
          name: def.lifecycle
        },
        log: _log["default"]
      });
    }

    var fn = fns[name];

    if (fn) {
      (0, _uiInternal.sandbox)(fn, {
        args: [value, previousValue, hostVariables[_symbols.HELPERS_SCHEDULED]],
        dispatch: dispatch,
        host: host,
        location: LIFECYCLE,
        details: {
          name: def.lifecycle
        },
        log: _log["default"]
      });
    }
  };
}

function lifeCycleFnsToHandlers(configs, tagName) {
  var actionHandlers = {};

  var _loop = function _loop(k) {
    var def = lifeCycleFnsToHandlerMap[k];
    var configsCount = configs.length;
    var actionType = k;
    actionHandlers[actionType] = [];

    var _loop2 = function _loop2(i) {
      var config = configs[i];

      if (actionType === _constants.COMPONENT_PROPERTY_CHANGED) {
        var onPropertyChangeEffect = createOnPropertyChangeHandler(config, def);

        if (onPropertyChangeEffect) {
          actionHandlers[actionType].push((0, _normalizeActionHandler.normalizeActionHandler)({
            effect: onPropertyChangeEffect,
            type: 'effect'
          }, tagName));
        }
      } else {
        var fn = config[def.lifecycle];

        if (typeof fn === 'function') {
          actionHandlers[actionType].push((0, _normalizeActionHandler.normalizeActionHandler)({
            effect: function effect(_ref2) {
              var dispatch = _ref2.dispatch,
                  action = _ref2.action,
                  host = _ref2.action.payload.host;
              var wrappedDispatch = (0, _utils.wrapDispatch)(host, action);
              var helpers = wrappedDispatch.dispatch;
              Object.assign(helpers, wrappedDispatch);
              (0, _uiInternal.sandbox)(fn, {
                args: config.options ? [host, helpers, config.options] : [host, helpers],
                dispatch: dispatch,
                host: host,
                location: LIFECYCLE,
                details: {
                  name: def.lifecycle
                },
                log: _log["default"]
              });
            },
            type: 'effect'
          }, tagName));
        }
      }
    };

    for (var i = 0; i < configsCount; i++) {
      _loop2(i);
    }
  };

  for (var k in lifeCycleFnsToHandlerMap) {
    _loop(k);
  }

  return actionHandlers;
}
//# sourceMappingURL=lifeCycleFnsToHandlers.js.map