/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = parseProperties;

var _constants = require("../../constants");

var _isValuePropertySelector = _interopRequireDefault(require("./isValuePropertySelector"));

var _uiInternal = require("@servicenow/ui-internal");

var _log = _interopRequireDefault(require("../../log"));

var _types = _interopRequireDefault(require("./types"));

var _refProperty = require("./refProperty");

var _parse = _interopRequireDefault(require("./parse"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Parse any property that is configured with `unstableParse: true`, or any property that has a
 * defined schema with a parseable type
 */
function parseProperties(properties) {
  var defs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _ref = arguments.length > 2 ? arguments[2] : undefined,
      host = _ref.host,
      dispatch = _ref.dispatch;

  var parsedProperties = {};

  for (var k in properties) {
    var value = properties[k];
    var unstableParse = defs[k] && defs[k][_constants.PARSE_PROP_CONF_KEY];
    var schemaType = defs[k] && defs[k].schema && defs[k].schema.type;

    var isValidParseType = _types["default"].includes(schemaType);

    var resolvedValue = defs[k] && defs[k].resolveRef ? (0, _refProperty.parseRefProperty)(host, k, value) : value;
    var shouldUnstableParse = (unstableParse || _types["default"].includes(schemaType)) && !(0, _isValuePropertySelector["default"])(resolvedValue) && typeof resolvedValue === 'string';

    if (shouldUnstableParse && isValidParseType && !host.hasAttribute((0, _uiInternal.kebabCase)(k))) {
      var component = "<".concat((0, _uiInternal.toLower)(host.tagName), " />");
      (0, _log["default"])("For property ".concat(k, " on component ").concat(component, ": expected to be type ").concat(schemaType, " but received string. ").concat(_constants.PACKAGE_NAME, " will attempt to parse this value, but you should set this to the correct type."), {
        level: _uiInternal.allLogTypes.WARN,
        host: host,
        origin: 'parseProperties'
      });
    }

    parsedProperties[k] = shouldUnstableParse ? (0, _uiInternal.sandbox)(_parse["default"], {
      args: [resolvedValue, schemaType, k, host.tagName],
      host: host,
      dispatch: dispatch,
      location: _uiInternal.locations.PROPERTY,
      details: {
        parse: true,
        name: k
      },
      log: _log["default"]
    }) : resolvedValue;
  }

  return parsedProperties;
}
//# sourceMappingURL=properties.js.map