/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { createEventActionDetails } from './utils';
import { CONFIG, HOIST_COMPOSED_PATH } from '../symbols';
import { SEISMIC_HOIST_TAG_NAME } from '../constants';
import { getHostVariables } from '../utils';
const allowedPropertyUpdates = ['payload', 'error', 'meta'];

function getPath(meta, host) {
  if (meta.event.path) return meta.event.path;
  const path = [];
  let el = host;
  let parentNode = el.parentNode || el.host;

  while (parentNode) {
    path.push(el);
    el = parentNode;
    parentNode = el.parentNode || el.host;
  }

  return [...path, document, window];
}

function getComposedPath(meta, host) {
  const hoistComposedPath = meta[HOIST_COMPOSED_PATH];
  if (!hoistComposedPath) return meta.event.composedPath();
  const composedPath = [];

  for (let i = 0; i < hoistComposedPath.length; i++) {
    const el = hoistComposedPath[i];
    if (el.tagName === SEISMIC_HOIST_TAG_NAME) break;
    composedPath.push(el);
  }

  return [...composedPath, ...getPath(meta, host)];
}

export function addActionHandler(targetEl, targetSelector, host, type, capture, eventHandler, handler) {
  const hostVariables = getHostVariables(host);

  const listener = event => {
    const config = hostVariables[CONFIG];
    const detail = createEventActionDetails(type, host, event, capture, eventHandler);
    handler(targetEl, targetSelector, host, type, new Proxy({ ...detail,

      stopPropagation() {
        return event.stopPropagation();
      }

    }, {
      get(action, key) {
        const {
          payload,
          meta
        } = action;

        if (key === 'defaultPrevented') {
          return payload && payload.event ? payload.event.defaultPrevented : event.defaultPrevented;
        }

        if (key === 'preventDefault') {
          return () => payload && payload.event ? payload.event.preventDefault() : event.preventDefault();
        }

        if (key === 'composedPath') return () => getComposedPath(meta, host);
        return action[key];
      },

      set(action, key, value) {
        if (allowedPropertyUpdates.includes(key)) {
          action[key] = value; // eslint-disable-line no-param-reassign

          return true;
        }

        return false;
      }

    }));
    if (config && config.shouldStopPropagation[type]) event.stopPropagation();
  };

  targetEl.addEventListener(type, listener, capture);
  return function removeActionHandler() {
    targetEl.removeEventListener(type, listener, capture);
  };
}
//# sourceMappingURL=customEventActionHandler.js.map