/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getConfigItem, configConstants } from '@servicenow/ui-config';
import { memoize } from '@servicenow/ui-internal';
import { getInteractionId } from '@servicenow/ui-metrics';
import { DISPATCH_EVENT, SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, COMPONENT_PROPERTY_UPDATE_REQUESTED } from '../constants';
import { COMPONENT_ID, PRESENTATIONAL } from '../symbols';
import { getHostVariables } from '../utils';
import { schedule, types } from '../scheduler';
import { applyRuntimeActionHandlers } from './utils';
const {
  ENABLE_ACTION_DISPATCHED_HANDLING
} = configConstants;
const shouldApplyRuntimeActionHandlers = memoize(() => getConfigItem(ENABLE_ACTION_DISPATCHED_HANDLING) === true, {
  profileName: 'shouldApplyRuntimeActionHandlers'
});
const {
  DISPATCH,
  IMMEDIATE
} = types;

function addEventGetters(event) {
  // FIXME: is there a better, faster way than this?
  Object.defineProperties(event.detail.meta, {
    event: {
      value: event,
      enumerable: true
    },
    path: {
      get() {
        return event.composedPath().map(el => {
          const variables = getHostVariables(el);
          return variables && variables[COMPONENT_ID];
        }).filter(componentId => componentId);
      }

    }
  });
  return event;
}

function dispatchLegacyEvent(host, detail) {
  const event = new CustomEvent(DISPATCH_EVENT, {
    composed: true,
    bubbles: true,
    detail
  });
  host.dispatchEvent(addEventGetters(event));
}

export function customEventDispatcher(host, {
  type,
  payload,
  meta,
  error
}) {
  const hostVariables = getHostVariables(host);
  const presentational = hostVariables[PRESENTATIONAL];
  const shouldBubble = meta.internal && type !== COMPONENT_PROPERTY_UPDATE_REQUESTED ? false : true;
  const event = new CustomEvent(type, {
    composed: shouldBubble,
    bubbles: shouldBubble,
    cancelable: true,
    detail: {
      type,
      payload,
      error,
      meta
    }
  });
  schedule({
    fn() {
      if (process.env.UNSAFE_SEISMIC_EMIT_CUSTOM_EVENT) {
        dispatchLegacyEvent(host, {
          type,
          payload,
          error,
          // spread or Object.defineProperties fails because the meta reference is the same
          meta: { ...meta
          }
        });
      }

      if (!presentational && shouldApplyRuntimeActionHandlers()) applyRuntimeActionHandlers(host, type);
      return host.dispatchEvent(addEventGetters(event));
    },

    meta: {
      el: host,
      interactionId: getInteractionId(meta)
    },
    type: meta[SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] ? IMMEDIATE : DISPATCH
  });
  return event;
}
//# sourceMappingURL=customEventDispatcher.js.map