/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getInteractionId } from '@servicenow/ui-metrics';
import { dispatchError, locations } from '@servicenow/ui-internal';
import log from '../log';
import { effectRunner } from '../effects';

function sandboxEffect(effect, dispatch, action, host) {
  const {
    type: actionType,
    meta: {
      effectType = ''
    },
    payload = {}
  } = action;
  let location = '';
  let details = {};

  if (effectType === locations.EVENT) {
    const {
      event
    } = payload;
    const {
      type
    } = event;
    location = locations.EVENT;
    details = {
      type,
      event
    };
  } else {
    location = locations.EFFECT;
    details = {
      actionType
    };
  }

  return function sandboxedEffect(...args) {
    try {
      // there can be multiple handlers for a given action type defined
      // by multiple behaviors; we only want to apply the options specific
      // to the effects for a single behavior, type, and effect combination
      const {
        options
      } = effect;

      if (options) {
        const {
          payload,
          meta
        } = args[args.length - 1].action;
        if (meta.options === null || meta.options === undefined) meta.options = options;

        if (payload === null || payload === undefined) {
          args[args.length - 1].action.payload = {
            options
          }; // eslint-disable-line no-param-reassign
        } else if (typeof payload === 'object') {
          // TODO: payload.options could have be applied from a previous behavior
          // or it could be part of the payload dispatched; not sure what to log
          // or how to properly handle this scenario. if options were part of the metadata
          // instead it would solved the problem.
          // if (typeof payload.options === 'object')
          payload.options = options;
        }
      }

      const transactionCtx = {
        interactionId: getInteractionId(action.meta),
        host,
        type: action.type
      };
      effectRunner(effect, transactionCtx, ...args);
    } catch (error) {
      dispatchError(error, {
        dispatch,
        host,
        location,
        details,
        metadata: {
          internal: true
        },
        log
      });
    }
  };
}

export { sandboxEffect };
//# sourceMappingURL=sandboxEffect.js.map