/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * Maintains task insert order but (when dedupe==true) will skip earlier tasks
 * if there's another task for the same element enqueued later
 */
export default class Queue {
  constructor(dedupe, doNotDedup = [], whenQueueIsEmptyForElementCb) {
    this._els = new WeakMap();
    this._tasks = [];
    this._dedupe = dedupe;
    this._doNotDedup = doNotDedup;
    this._whenQueueIsEmptyForElementCb = whenQueueIsEmptyForElementCb;
  }

  push(task) {
    const {
      el
    } = task.meta;

    this._els.set(el, this.sizeForElement(el) + 1);

    this._tasks.push(task);
  } // this value may be a little misleading because it doesn't account for
  // possible task duplicates


  get length() {
    return this._tasks.length;
  }

  sizeForElement(el) {
    return this._els.get(el) || 0;
  }

  shift() {
    while (this._tasks.length) {
      const task = this._tasks.shift();

      const {
        el
      } = task.meta;

      const existingCount = this._els.get(el);

      if (existingCount === 1) {
        this._els.delete(el);

        if (this._whenQueueIsEmptyForElementCb) this._whenQueueIsEmptyForElementCb(el);
      } else {
        this._els.set(el, existingCount - 1);
      }

      if (existingCount === 1 || !this._dedupe || this._doNotDedup.indexOf(task.type) !== -1 || !el.isConnected) return task;
    }
  }

}
//# sourceMappingURL=queue.js.map