/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import log from '../logValidationError';

function getPath(pathPrefix, dataPath) {
  if (!pathPrefix && !dataPath) return '';
  if (!pathPrefix && dataPath.startsWith('.')) return `${dataPath.substr(1)} `;
  return `${pathPrefix}${dataPath} `;
}
/**
 * Run a validator against a value to determine its validity. If it is not valid, log an error.
 * @private
 * @param {Function} validator The validator built from one of the methods in ./build
 * @param {any} value The value to run the schema validation against
 * @param {Object} options
 * @param {string} options.pathPrefix Name of the value being validated. Optional
 * @param {string} options.errorPrefix Prefix for the error message
 * @returns {boolean} Whether the value passed validation
 */


function validateSchema(validator, value, {
  pathPrefix = '',
  errorPrefix = '',
  shouldLog = true
} = {}) {
  const isValid = validator(value);

  if (!isValid) {
    // If there are errors, only report the first one
    const [{
      dataPath,
      message,
      params: {
        level = 'error'
      } = {}
    }] = validator.errors;
    const path = getPath(pathPrefix, dataPath);
    const errorMessage = `${path}${message}`;
    if (shouldLog) log(`${errorPrefix}${errorMessage}`, {
      level,
      origin: 'validateSchema'
    });
  }

  return isValid;
}

export default validateSchema;
//# sourceMappingURL=validateSchema.js.map