/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { schedule, types, countHighPriorityTasks, countMediumPriorityTasks, countAllHighPriorityTasks, countAllMediumPriorityTasks } from './scheduler';
import log from './log';
import { getHostPendingNetworkActionCount, getPendingNetworkActionCount } from '@servicenow/ui-internal';
import { getConfigItem, configConstants } from '@servicenow/ui-config';
const {
  WHEN_COMPONENT_IS_IDLE_TIMEOUT,
  WHEN_COMPONENT_IS_IDLE_THROTTLE,
  WHEN_COMPONENT_IS_IDLE_MIN_INTERVAL
} = configConstants;
const timeout = getConfigItem(WHEN_COMPONENT_IS_IDLE_TIMEOUT);
const throttle = getConfigItem(WHEN_COMPONENT_IS_IDLE_THROTTLE);
const interval = getConfigItem(WHEN_COMPONENT_IS_IDLE_MIN_INTERVAL);
const {
  WHEN_COMPONENT_IS_IDLE,
  WHEN_ALL_COMPONENTS_IDLE
} = types;

const check = el => {
  return new Promise(resolve => {
    schedule({
      fn() {
        resolve(getHostPendingNetworkActionCount(el) + countHighPriorityTasks(el) + countMediumPriorityTasks(el));
      },

      type: WHEN_COMPONENT_IS_IDLE,
      meta: {
        el,
        interactionId: 'componentIdleCheck'
      }
    });
  });
};

const pause = () => new Promise(resolve => {
  setTimeout(() => resolve(), throttle);
});

const checkAll = () => {
  return new Promise(resolve => {
    schedule({
      fn() {
        resolve(getPendingNetworkActionCount() + countAllMediumPriorityTasks() + countAllHighPriorityTasks());
      },

      type: WHEN_ALL_COMPONENTS_IDLE,
      meta: {
        el: {},
        interactionId: 'allComponentsIdleCheck'
      }
    });
  });
};

export function whenAllComponentsIdle() {
  return new Promise(resolve => {
    schedule({
      async fn() {
        let busy = true;
        let didPause = false;
        const timeoutId = setTimeout(() => {
          busy = false;
          resolve(didPause);
        }, timeout);

        while (busy) {
          busy = await checkAll();

          if (busy) {
            didPause = true;
            await pause();
          }
        }

        clearTimeout(timeoutId);
        resolve(didPause);
      },

      type: WHEN_ALL_COMPONENTS_IDLE,
      meta: {
        el: {},
        interactionId: 'whenAllComponentsIdle'
      }
    });
  });
}
export default function whenComponentIsIdle(el) {
  return new Promise(resolve => {
    schedule({
      async fn() {
        let busy = true;
        const timeoutId = setTimeout(() => {
          log('whenComponentIsIdle timed out.', {
            host: el,
            origin: 'whenComponentIsIdle'
          });
          busy = false;
          resolve(el);
        }, timeout);

        while (busy) {
          const start = Date.now();
          busy = await check(el);
          const end = Date.now(); // throttle in case of network activity, i.e. something is pending, but component is otherwise idle
          // if not there will be numerous unnecessary checks while waiting for network request to complete

          if (busy && end - start < interval) await pause();
        }

        clearTimeout(timeoutId);
        resolve(el);
      },

      type: WHEN_COMPONENT_IS_IDLE,
      meta: {
        el,
        interactionId: 'whenComponentIsIdle'
      }
    });
  });
}
//# sourceMappingURL=whenComponentIsIdle.js.map